# Get content from the config.json
$config = Get-Content -Path .\config.json | ConvertFrom-Json

$xu_location = $config.xu_location
$xu_server = $config.xu_server
$xu_port = $config.xu_port
$end_of_upload_api = $config.end_of_upload_api

$extraction_start_date = $config.extraction_start_date
$extraction_end_date = $config.extraction_end_date
$language = $config.language
$exchange_rate_type = $config.exchange_rate_type

# Create a new log file for every run
$logFile = ".\extraction.log"

if (Test-Path -Path .\$logFile -PathType Leaf) {
    Remove-Item .\$logFile
}

# Function for appending text to the log file
function WriteLog
{
Param ([string]$logText)
$timestamp = (Get-Date).toString("dd/MM/yyyy HH:mm:ss")
Add-content $logFile -value "$timestamp $logText"
Write-Output $logText
}

# Function for checking the error code of extractions
function CheckExtractionError
{
Param ([string]$extraction)
if ($LASTEXITCODE -ne 0) {
    WriteLog("$extraction failed with return code: $LASTEXITCODE")
    Set-Variable -Name "extraction_failed" -Value 1 -Scope global
}
}

# If at least one extraction fails, do not upload to blob storage.
Set-Variable -Name "extraction_failed" -Value 0 -Scope global

# Extraction commands
# Transactional tables
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_AFKO" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('PP_AFKO')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_AFPO" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('PP_AFPO')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_AFRU" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('PP_AFRU')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_AFVC" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('PP_AFVC')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_AUFK" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('PP_AUFK')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_JCDS" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('PP_JCDS')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_JEST" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('PP_JEST')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_MSEG" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('PP_MSEG')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_CDHDR" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('PP_CDHDR')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_CDPOS" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('PP_CDPOS')

# Master data tables 
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_ADCP"
CheckExtractionError('PP_ADCP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_ADRP"
CheckExtractionError('PP_ADRP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_MARC"
CheckExtractionError('PP_MARC')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_MARM"
CheckExtractionError('PP_MARM')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_MBEW"
CheckExtractionError('PP_MBEW')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_T001"
CheckExtractionError('PP_T001')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_T001L"
CheckExtractionError('PP_T001L')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_T001W"
CheckExtractionError('PP_T001W')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_T003P" -o "language=$language"
CheckExtractionError('PP_T003P')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_T006A" -o "language=$language"
CheckExtractionError('PP_T006A')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_T156T" -o "language=$language"
CheckExtractionError('PP_T156T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_T156"
CheckExtractionError('PP_T156')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_TCA02" -o "language=$language"
CheckExtractionError('PP_TCA02')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_TGSBT" -o "language=$language"
CheckExtractionError('PP_TGSBT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_TJ02T" -o "language=$language"
CheckExtractionError('PP_TJ02T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_TKA01"
CheckExtractionError('PP_TKA01')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_USR02"
CheckExtractionError('PP_USR02')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_USR21"
CheckExtractionError('PP_USR21')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_TTZCU"
CheckExtractionError('PP_TTZCU')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_TTZD"
CheckExtractionError('PP_TTZD')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_TTZDF"
CheckExtractionError('PP_TTZDF')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_TTZDV"
CheckExtractionError('PP_TTZDV')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_TTZR"
CheckExtractionError('PP_TTZR')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_TTZZ"
CheckExtractionError('PP_TTZZ')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_TCURX"
CheckExtractionError('PP_TCURX')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_TCURF" -o "exchange_rate_type=$exchange_rate_type"
CheckExtractionError('PP_TCURF')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "PP_TCURR" -o "exchange_rate_type=$exchange_rate_type"
CheckExtractionError('PP_TCURR')

# Only ingest data if extraction is successful.
if ($extraction_failed) {
    Exit
} else {
    Invoke-WebRequest -Method POST -Uri $end_of_upload_api
}
